/*
 * MSU CUDA Course Examples and Exercises.
 *
 * Copyright (c) 2011 Dmitry Mikushin
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising 
 * from the use of this software.
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it freely,
 * without any restrictons.
 */

#include "test.h"

#include <assert.h>

__device__ float user_function(int index, float* array, int length)
{
	// return some function of cell_number and vars_array
	return 1.1f * (float)(index + 1) * array[index % length];
}

__global__ void user_function_ptr(
	external_device_func_t* dev_func_ptr)
{
	*dev_func_ptr = user_function;
}

int main(int argc, char** argv)
{
	external_device_func_t* func;
	cudaError_t status = cudaMalloc((void**)&func, sizeof(external_device_func_t));
	assert(status == cudaSuccess);
	user_function_ptr<<<1, 1>>>(func);

	// Create test class instance, specifying config file
	// and user data initialization function.
	Test t("task.conf", func);

	// Fill device vector.
	t.FillVector();

	// Print filled device vector
	t.PrintVector();

	status = cudaFree(func);
	assert(status == cudaSuccess);

	return 0;
}

