/*
 * MSU CUDA Course Examples and Exercises.
 *
 * Copyright (c) 2011 Dmitry Mikushin
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising 
 * from the use of this software.
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it freely,
 * without any restrictons.
 */

#include <ImageMagick/magick/MagickCore.h>

#define SZCELL 10

// Perform simple results difference visualization.
void draw_diffs(int nx, int ny, float* diffs, float eps)
{
	// Create image info template.
	ExceptionInfo exception;
	ImageInfo* iinfo = AcquireImageInfo();
	
	// Create background.
	MagickPixelPacket background;
	GetExceptionInfo(&exception);
	QueryMagickColor("white", &background, &exception);

	// Create new image.
	int width = nx * SZCELL + 1, height = ny * SZCELL + 1;
	Image* image = NewMagickImage(iinfo, width, height, &background);

	// Set image filename.
	sprintf(image->filename, "pattern2d.png");

	DrawInfo* dinfo = CloneDrawInfo(NULL, NULL);

	// Drawing command template.
	char rectcmd[100];
	const char* rectfmt = "rectangle %f,%f %f,%f";
	
	for (int j = 0; j < ny; j++)
		for (int i = 0; i < nx; i++)
	{
		// Select cell color, depending on diff.
		if (diffs[i + j * nx] >= eps)
			QueryColorDatabase("red", &dinfo->fill, &exception);
		else
			QueryColorDatabase("green", &dinfo->fill, &exception);

		// Contruct a rectangle drawing command.
		sprintf(rectcmd, rectfmt,
			(float)(i * SZCELL + 1), (float)(j * SZCELL + 1),
			(float)((i + 1) * SZCELL - 1), (float)((j + 1) * SZCELL - 1));

		// Query command for execution.
		dinfo->primitive = rectcmd;
		DrawImage(image, dinfo);
	}

	// Write image to file.
	WriteImage(iinfo, image);
	
	// Release used resources.
	DestroyImage(image);
	DestroyImageInfo(iinfo);
}

