/*
 * MSU CUDA Course Examples and Exercises.
 *
 * Copyright (c) 2011 Dmitry Mikushin
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising 
 * from the use of this software.
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it freely,
 * without any restrictons.
 */

#include "test.h"

#include <assert.h>
#include <iostream>
#include <fstream>

using namespace std;

__device__ float user_function(int index, float* array, int length)
{
	// return some function of cell_number and vars_array
	return 1.1f * (float)(index + 1) * array[index % length];
}

class Transform : public thrust::binary_function<float, int, float>
{
	int m_n;
	float* m_array;

public:
	Transform(float* array, int n) : m_array(array), m_n(n) { }

	__device__ float operator()(const float& u_current, const int& k) const
	{
		// Call user-specified function.
		return user_function(k, m_array, m_n);
	}
};

Test::Test(char* filename)
{
	// Open config file.
	ifstream in;
	in.open(filename);

	// Fetch other value from file, denoting the length of array.
	in >> m_n;

	// Create device vector for GPU processing.
	device_array = thrust::device_vector<float>(m_n);

	// Read array values.
	for (int i = 0; i < m_n; ++i)
	{
		float value; in >> value;
		device_array[i] = value;
	}
}

void Test::FillVector()
{
	// Create transform and iterator.
	Transform tr(thrust::raw_pointer_cast(&device_array[0]), m_n);
	thrust::counting_iterator<int> it(0);

	// Perform transformation on device vector.
	thrust::transform(device_array.begin(), device_array.end(),
		it, device_array.begin(), tr);
}

void Test::PrintVector()
{
	// Simply print array values one by one.
	cout << "Printing" << endl;
	for (int i = 0; i < m_n; ++i)
		cout << device_array[i] << " ";
	cout << endl;
}

int main(int argc, char** argv)
{
	// Create test class instance, specifying config file
	// and user data initialization function.
	Test t("task.conf");

	// Fill device vector.
	t.FillVector();

	// Print filled device vector
	t.PrintVector();

	return 0;
}

